/***************************************************************************
 *            Timer.h
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef TIMER_H
#define TIMER_H

#include "boost/date_time/posix_time/posix_time_types.hpp"

using namespace boost::posix_time;

class Timer
{
	ptime begin, end;
	bool started;

public:
	Timer() : begin(ptime(second_clock::local_time())), end(), started(true) {};
	
	void start();
	void stop();
	void reset();
	
	long elapsed();
	
};

inline void Timer::start()
{
	if (!started)
	{
		begin = ptime(second_clock::local_time());
		end = ptime();		
		started = true;
	}
}

inline void Timer::stop()
{
	if (started)
	{
		end = ptime(second_clock::local_time());
		started = false;
	}
}

inline void Timer::reset()
{
	if (started)
	{
		begin = ptime(second_clock::local_time());
	}
}

inline long Timer::elapsed(){
	if (started)
	{
		ptime current(second_clock::local_time());
		return time_period(begin, current).length().total_seconds();
	}
	
	else
	{
		return time_period(begin, end).length().total_seconds();
	}
}

#endif
