//           ClientsInfoNoteBookTab.cc
//  Mon Sep 10 00:34:57 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

 
#include "ClientsInfoNoteBookTab.h"
#include "XMLReader.h"
#include "XMLWriter.h"

#include <iostream>
#include <numeric>

#include "TorrentInfoTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include "Lince/Core.h"
#include "Lince/SessionManager.h"
#include "Lince/Utils.h"

using namespace Lince;

ClientsInfoNoteBookTab::ClientsInfoNoteBookTab(TorrentInfoTab *_tit):
	nameTab(_("Clients")),
	ScrolledWindow()
{
	// Add the TreeView, inside the ScrolledWindow, with the button underneath:
	// Only show the scrollbars when they are necessary:
	add(clientsTreeView);
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
		
	// Create the Tree model:
	refTreeModel = Gtk::ListStore::create(clientsColumns);
	clientsTreeView.set_model(refTreeModel);
	clientsTreeView.set_rules_hint();
	
	// Add the TreeView's view columns:
	clientsTreeView.append_column(_("Flag"), clientsColumns.m_client_flag);
	clientsTreeView.append_column("IP", clientsColumns.m_client_ip);
	clientsTreeView.append_column(_("Country"), clientsColumns.m_client_country);
	clientsTreeView.append_column(_("Client"), clientsColumns.m_client_name);
	clientsTreeView.append_column(_("T"), clientsColumns.m_client_connection_type);
	
	Gtk::CellRendererProgress* cell = Gtk::manage(new Gtk::CellRendererProgress);
	int cols_count = clientsTreeView.append_column(_("Percentage"), *cell);
	Gtk::TreeViewColumn* pColumn = clientsTreeView.get_column(cols_count-1);
	if(pColumn){
#ifdef GLIBMM_PROPERTIES_ENABLED
				pColumn->add_attribute(cell->property_value(), clientsColumns.m_client_percentage);
#else
				pColumn->add_attribute(*cell, _("value"), clientsColumns.m_percentage_value);
#endif
	}
	
	clientsTreeView.append_column(_("Up. Speed"), clientsColumns.m_client_up_speed);
	clientsTreeView.append_column(_("Down. Speed"), clientsColumns.m_client_down_speed);
	clientsTreeView.append_column(_("State"), clientsColumns.m_client_state);
	clientsTreeView.append_column(_("Up. Data"), clientsColumns.m_client_up_data);
	clientsTreeView.append_column(_("Down. Data"), clientsColumns.m_client_down_data);
		
	for(guint i = 0; i < 11; i++){
		Gtk::TreeView::Column* pColumn = clientsTreeView.get_column(i);
		Gtk::CellRenderer* cColumn = clientsTreeView.get_column_cell_renderer(i);
		
		//pColumn->set_reorderable();		
		pColumn->set_resizable();
				
		if (i >= 6 && i <= 7){
			if (i == 6) pColumn->set_sort_column(clientsColumns.m_up_speed);
			if (i == 7) pColumn->set_sort_column(clientsColumns.m_down_speed);
			pColumn->set_alignment(1);
			cColumn->property_xalign() = 1.0;
		}
		
		switch(i){
			case 0:
				pColumn->set_sort_column(clientsColumns.m_client_country);
				break;
			case 1:
				pColumn->set_sort_column(clientsColumns.m_client_ip);
				break;
			case 2:
				pColumn->set_sort_column(clientsColumns.m_client_country);
				break;
			case 3:
				pColumn->set_sort_column(clientsColumns.m_client_name);
				break;
			case 4:
				pColumn->set_sort_column(clientsColumns.m_client_connection_type);
				break;
			case 5:
				pColumn->set_sort_column(clientsColumns.m_client_percentage);
				pColumn->set_expand();
				pColumn->set_alignment(0.5);
				cColumn->property_xalign() = 0.0;
				break;
			case 8:
				pColumn->set_sort_column(clientsColumns.m_client_state);
				break;
			case 9:
				pColumn->set_sort_column(clientsColumns.m_up_data);
				break;
			case 10:
				pColumn->set_sort_column(clientsColumns.m_down_data);
				break;
		}
	}
	
	clientsTreeView.columns_autosize();
	
	clientsTreeView.signal_button_press_event().connect_notify(sigc::mem_fun(*this,  &ClientsInfoNoteBookTab::mouse_button_press_event));
	
	m_refActionGroup = Gtk::ActionGroup::create();
	m_refActionGroup->add( Gtk::Action::create("ContextMenu", "Context Menu"));
	m_refActionGroup->add( Gtk::Action::create("BanClient", _("Ban Client")) );
	
	m_refActionGroup->add( Gtk::Action::create("forever", _("Forever")), sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::banClient), -1) );
	m_refActionGroup->add( Gtk::Action::create("5m", _("5 minutes")), sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::banClient), 300) );
	m_refActionGroup->add( Gtk::Action::create("15m", _("15 minutes")), sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::banClient), 900) );
	m_refActionGroup->add( Gtk::Action::create("45m", _("45 minutes")), sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::banClient), 2700) );
	m_refActionGroup->add( Gtk::Action::create("1h", _("1 hour")), sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::banClient), 3600) );
	m_refActionGroup->add( Gtk::Action::create("2h", _("2 hours")), sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::banClient), 7200) );
	
	
	m_refActionGroup->add( Gtk::Action::create("Columns", _("Columns")) );
	
	showFlag = Gtk::ToggleAction::create("ShowFlag", _("Show Flag"), "Flag");
	showFlag->set_active();
	m_refActionGroup->add( showFlag, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 0));
	
	showIp = Gtk::ToggleAction::create("ShowIp", _("Show Ip"), "Ip");
	showIp->set_active();
	m_refActionGroup->add( showIp, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 1));
	
	showCountry = Gtk::ToggleAction::create("ShowCountry", _("Show Country"), "Country");
	showCountry->set_active();
	m_refActionGroup->add( showCountry, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 2));
	
	showClient = Gtk::ToggleAction::create("ShowClient", _("Show Client"), "Client");
	showClient->set_active();
	m_refActionGroup->add( showClient, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 3));
	
	showConnectionType = Gtk::ToggleAction::create("ShowConnectionType", _("Show Connection Type"), "ConnectionType");
	showConnectionType->set_active();
	m_refActionGroup->add( showConnectionType, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 4));
	
	showPercentage = Gtk::ToggleAction::create("ShowPercentage", _("Show Percentage"), "Percentage");
	showPercentage->set_active();
	m_refActionGroup->add( showPercentage, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 5));
	
	showUSpeed = Gtk::ToggleAction::create("ShowUSpeed", _("Show Up. Speed"), "Up Speed");
	showUSpeed->set_active();
	m_refActionGroup->add( showUSpeed, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 6));
	
	showDSpeed = Gtk::ToggleAction::create("ShowDSpeed", _("Show Down. Speed"), "Down Speed");
	showDSpeed->set_active();
	m_refActionGroup->add( showDSpeed, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 7));
	
	showState = Gtk::ToggleAction::create("ShowState", _("Show State"), "State");
	showState->set_active();
	m_refActionGroup->add( showState, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 8));
	
	showUData = Gtk::ToggleAction::create("ShowUData", _("Show Up. Data"), "Up Data");
	showUData->set_active();
	m_refActionGroup->add( showUData, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 9));
	
	showDData = Gtk::ToggleAction::create("ShowDData", _("Show Down. Data"), "Down Data");
	showDData->set_active();
	m_refActionGroup->add( showDData, sigc::bind<int> (sigc::mem_fun(this, &ClientsInfoNoteBookTab::hideColumn), 10));
	
	refUIManager = Gtk::UIManager::create();
	refUIManager->insert_action_group(m_refActionGroup);
	
	Glib::ustring ui_info =
    "<ui>"
    "  <popup name='PopupMenu'>"
	"    <menu action='BanClient'>"
	"    <menuitem action='forever'/>"
	"    <separator/>"
	"    <menuitem action='5m'/>"
	"    <menuitem action='15m'/>"
	"    <menuitem action='45m'/>"
	"    <menuitem action='1h'/>"
	"    <menuitem action='2h'/>"
	"	 </menu>"
	"    <separator/>"
	"	 <menu action='Columns'>"
	"    <menuitem action='ShowFlag'/>"
	"    <menuitem action='ShowIp'/>"
	"    <menuitem action='ShowCountry'/>"
	"    <menuitem action='ShowClient'/>"
	"    <menuitem action='ShowConnectionType'/>"
	"    <menuitem action='ShowPercentage'/>"
	"    <menuitem action='ShowUSpeed'/>"
	"    <menuitem action='ShowDSpeed'/>"
	"    <menuitem action='ShowState'/>"
	"    <menuitem action='ShowUData'/>"
	"    <menuitem action='ShowDData'/>"
	"	 </menu>"
    "  </popup>"
    "</ui>";
	
	
	n_refUIManager = Gtk::UIManager::create();
	n_refUIManager->insert_action_group(m_refActionGroup);
		
	Glib::ustring n_ui_info =
    "<ui>"
    "  <popup name='PopupMenu'>"
	"	 <menu action='Columns'>"
	"    <menuitem action='ShowFlag'/>"
	"    <menuitem action='ShowIp'/>"
	"    <menuitem action='ShowCountry'/>"
	"    <menuitem action='ShowClient'/>"
	"    <menuitem action='ShowConnectionType'/>"
	"    <menuitem action='ShowPercentage'/>"
	"    <menuitem action='ShowUSpeed'/>"
	"    <menuitem action='ShowDSpeed'/>"
	"    <menuitem action='ShowState'/>"
	"    <menuitem action='ShowUData'/>"
	"    <menuitem action='ShowDData'/>"
	"	 </menu>"
    "  </popup>"
    "</ui>";
	
#ifdef GLIBMM_EXCEPTIONS_ENABLED
	try{
		refUIManager->add_ui_from_string(ui_info);
		n_refUIManager->add_ui_from_string(n_ui_info);
	}
	catch(const Glib::Error& ex){
		std::cerr << "building menus failed: " <<  ex.what();
	}
#else
	std::auto_ptr<Glib::Error> ex;
	refUIManager->add_ui_from_string(ui_info, ex);
	if(ex.get()){
		std::cerr << "building menus failed: " <<  ex->what();
	}
	n_refUIManager->add_ui_from_string(n_ui_info, ex);
	if(ex.get()){
		std::cerr << "building menus failed: " <<  ex->what();
	}
#endif //GLIBMM_EXCEPTIONS_ENABLED
	
	pMenuPopup = Gtk::manage(dynamic_cast<Gtk::Menu*>(refUIManager->get_widget("/PopupMenu"))); 
	if(!pMenuPopup)
		g_warning("menu not found");
		
	n_pMenuPopup = Gtk::manage(dynamic_cast<Gtk::Menu*>(n_refUIManager->get_widget("/PopupMenu"))); 
	if(!n_pMenuPopup)
		g_warning("menu not found");
	
	tit = _tit;
}

void ClientsInfoNoteBookTab::updateRow(const libtorrent::peer_info & peer, Gtk::TreeModel::Row &row){
	float percentage = 100;
	Glib::ustring client_name(peer.client);
	
	if (!(peer.flags & libtorrent::peer_info::seed)){
		percentage = (std::accumulate(peer.pieces.begin(), peer.pieces.end(), 0) / (float) peer.pieces.size()) * 100;
	}
				
	if (!client_name.validate()){
		try{
			client_name.assign(Glib::locale_to_utf8(peer.client));
		}
		catch(Glib::ConvertError){
			client_name = "--";
		}
	}
	
	row[clientsColumns.m_client_ip] = peer.ip.address().to_string();
	row[clientsColumns.m_client_name] = client_name;
		
	if (peer.flags == libtorrent::peer_info::local_connection) row[clientsColumns.m_client_connection_type] = "L";
	else row[clientsColumns.m_client_connection_type] = "R";
	
	row[clientsColumns.m_client_percentage] = percentage;
	row[clientsColumns.m_client_up_speed] = size_to_string(peer.up_speed);
	row[clientsColumns.m_client_down_speed] = size_to_string(peer.down_speed);
	
	if (peer.flags & libtorrent::peer_info::interesting) row[clientsColumns.m_client_state] = _("Connected");
	else if (peer.flags & libtorrent::peer_info::choked) row[clientsColumns.m_client_state] = _("Waiting");
	else if (peer.flags & libtorrent::peer_info::remote_interested) row[clientsColumns.m_client_state] = _("Connected");
	else if (peer.flags & libtorrent::peer_info::remote_choked) row[clientsColumns.m_client_state] = _("Waiting");
	else if (peer.flags & libtorrent::peer_info::handshake) row[clientsColumns.m_client_state] = _("Connecting");
	else if (peer.flags & libtorrent::peer_info::connecting) row[clientsColumns.m_client_state] = _("Connecting");
	else if (peer.flags & libtorrent::peer_info::queued) row[clientsColumns.m_client_state] = _("Queued");
	else row[clientsColumns.m_client_state] = _("Unknown");
	
	if (!row[clientsColumns.has_flag])
    {
		char c[3];
        c[0] = peer.country[0];
        c[1] = peer.country[1];
        c[2] = '\0';
        Glib::ustring country = c;
        if (!country.empty() && country != "00" && country != "--" && country != "!!")
        {
			row[clientsColumns.m_client_country] = country;
			Glib::ustring flag = Glib::build_filename(FLAG_DIR, country.lowercase() + ".png");
            try
            {
				row[clientsColumns.m_client_flag] = Gdk::Pixbuf::create_from_file(flag);
				row[clientsColumns.has_flag] = true;
			}
			catch (Glib::Error& e)
			{
				g_warning(e.what().c_str());
			}
		}
	}
	
	row[clientsColumns.m_client_up_data] = size_to_string(peer.total_upload);
	row[clientsColumns.m_client_down_data] = size_to_string(peer.total_download);
	
	// Sort column
	row[clientsColumns.m_up_speed] = peer.up_speed;
	row[clientsColumns.m_down_speed] = peer.down_speed;
	row[clientsColumns.m_up_data] = peer.total_upload;
	row[clientsColumns.m_down_data] = peer.total_download;
	
	// Id Row
	row[clientsColumns.m_pid] = peer.pid;
}

void ClientsInfoNoteBookTab::updateClientsInfoNoteBookTab(Torrent *t){
	int i=0;
	
	std::vector<libtorrent::peer_info> peers;
	Gtk::TreeModel::Row row;
	Gtk::TreeModel::iterator iter;
	PeerMap::iterator peer_iter;
	
	t->get_peer_info(peers);
	
	// Fill peers map
	PeerMap peer_map;
    for (i = 0; i < peers.size(); i++){
		peer_map[peers[i].pid] = peers[i];
	}
	
	Gtk::SortType order;
    int col;
    refTreeModel->get_sort_column_id(col, order);
    refTreeModel->set_sort_column_id(Gtk::TreeSortable::DEFAULT_UNSORTED_COLUMN_ID, order);

	// Update used rows
	for (iter = refTreeModel->children().begin(); iter != refTreeModel->children().end(); ++iter){
		int i=0;
		row = *iter;
		row[clientsColumns.remove] = true;
		
		peer_iter = peer_map.find(row[clientsColumns.m_pid]);
		if (peer_iter != peer_map.end()){
			row[clientsColumns.remove] = false;
			
			updateRow(peer_iter->second, row);
			peer_map.erase(peer_iter);
			
			if (i > 20){
				i = 0;
				while (Gtk::Main::events_pending())
					Gtk::Main::iteration(false);
			}
			
			i++;
		}
	}
	
	// Erase disconnected peers 
	for (iter = refTreeModel->children().begin(); iter != refTreeModel->children().end();){
		row = *iter;
		
		if (row[clientsColumns.remove])
			iter = refTreeModel->erase(iter);
		else
			++iter;
	}
	
	// Add all new (remaining) peers 
    for (peer_iter = peer_map.begin(); peer_iter != peer_map.end(); ++peer_iter){
		row = *(refTreeModel->append());
		updateRow(peer_iter->second, row);
	}
	
	refTreeModel->set_sort_column_id(col, order);
}

bool ClientsInfoNoteBookTab::isRowSelected(){
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = clientsTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
	
	if(iter){ 
		return true;
	}
	
	return false;
}

void ClientsInfoNoteBookTab::unselectRows(void){
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = clientsTreeView.get_selection();
	
	refTreeSelection->unselect_all();
}


void ClientsInfoNoteBookTab::mouse_button_press_event(GdkEventButton* event){
	if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) && isRowSelected() ){
		if(pMenuPopup)
			pMenuPopup->popup(event->button, event->time);
	}
	else if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) && !isRowSelected() ){
		if(n_pMenuPopup)
			n_pMenuPopup->popup(event->button, event->time);
	}
	else if( (event->type == GDK_BUTTON_PRESS) && (event->button == 1) ){
		unselectRows();
	}
}

void ClientsInfoNoteBookTab::hideColumn(int value){
	Gtk::TreeView::Column* pColumn = clientsTreeView.get_column(value);
	
	if (pColumn->get_visible()) pColumn->set_visible(false);
	else pColumn->set_visible(true);
}

void ClientsInfoNoteBookTab::banClient(int ban_time){
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = clientsTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
	
	if(iter){
		Gtk::TreeModel::Row row = *iter;
		Glib::ustring ban_data = row[clientsColumns.m_client_ip] + "-" 
			+ row[clientsColumns.m_client_ip];
		
		tit->sendBanMessageToNoteBookManager(ban_data, ban_time);
		refTreeModel->erase(*iter);
	}
	
}

ClientsInfoNoteBookTab::~ClientsInfoNoteBookTab(){
}
