//           DataStatsTab.cc
//  Sat Dec 22 16:03:39 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include <sstream>

#include "DataStatsTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

DataStatsTab::DataStatsTab():
	nameTab(_("Transference")),
	dataWindow(false, 5)
{
	Gtk::Table *dataTable1, *dataTable2;
	Gtk::HBox *box1, *box2;
	Gtk::Frame *speedFrame, *sizeFrame, *dhtFrame;
	Gtk::Label *downloaded, *uploaded, *shared, *time;
	Gtk::Label *now, *session, *total;
	Gtk::Label *downloading, *shared2;
	Gtk::Label *total2, *downloaded2, *ratio;

	dataTable1 = Gtk::manage(new Gtk::Table(4, 5, true));
	dataTable2 = Gtk::manage(new Gtk::Table(3, 5, true));

	speedFrame = Gtk::manage(new Gtk::Frame());
	sizeFrame = Gtk::manage(new Gtk::Frame());

	box1 = Gtk::manage(new Gtk::HBox(false, 5));
	box2 = Gtk::manage(new Gtk::HBox(false, 5));

	downloaded = Gtk::manage(new Gtk::Label(_("Downloaded")));
	uploaded = Gtk::manage(new Gtk::Label(_("Uploaded")));
	shared = Gtk::manage(new Gtk::Label(_("Shared")));
	time = Gtk::manage(new Gtk::Label(_("Time")));

	now = Gtk::manage(new Gtk::Label(_("Now")));
	session = Gtk::manage(new Gtk::Label(_("Session")));
	total = Gtk::manage(new Gtk::Label(_("Total")));

	downloading = Gtk::manage(new Gtk::Label(_("Downloading")));
	shared2 = Gtk::manage(new Gtk::Label(_("Shared")));

	total2 = Gtk::manage(new Gtk::Label(_("Total")));
	downloaded2 = Gtk::manage(new Gtk::Label(_("Downloaded")));
	ratio = Gtk::manage(new Gtk::Label(_("Ratio")));

	now->property_xalign() = 0; session->property_xalign() = 0;
	total->property_xalign() = 0;

	downloading->property_xalign() = 0; shared2->property_xalign() = 0;

	downloaded->property_xalign() = 0; uploaded->property_xalign() = 0;
	shared->property_xalign() = 0; time->property_xalign() = 0;
	downloadedNow.property_xalign() = 0; downloadedSession.property_xalign() = 0;
	downloadedTotal.property_xalign() = 0; uploadedNow.property_xalign() = 0;
	uploadedSession.property_xalign() = 0; uploadedTotal.property_xalign() = 0;
	shareSession.property_xalign() = 0; shareTotal.property_xalign() = 0;
	timeSession.property_xalign() = 0; timeTotal.property_xalign() = 0;
	currentDownloadingSize.property_xalign() = 0; totalDownloadingSize.property_xalign() = 0;
	downloadingRatio.property_xalign() = 0; totalSharedSize.property_xalign() = 0;


	downloaded2->property_xalign() = 0;  total2->property_xalign() = 0;
	ratio->property_xalign() = 0;

	dataTable1->attach(*downloaded, 1, 2, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	dataTable1->attach(*uploaded, 2, 3, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	dataTable1->attach(*shared, 3, 4, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	dataTable1->attach(*time, 4, 5, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);

	dataTable1->attach(*now, 0, 1, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(*session, 0, 1, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(*total, 0, 1, 3, 4, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable1->attach(downloadedNow, 1, 2, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(downloadedSession, 1, 2, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(downloadedTotal, 1, 2, 3, 4, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable1->attach(uploadedNow, 2, 3, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(uploadedSession, 2, 3, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(uploadedTotal, 2, 3, 3, 4, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable1->attach(shareSession, 3, 4, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(shareTotal, 3, 4, 3, 4, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable1->attach(timeSession, 4, 5, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable1->attach(timeTotal, 4, 5, 3, 4, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable2->attach(*total2, 1, 2, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable2->attach(*downloaded2, 2, 3, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable2->attach(*ratio, 3, 4, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable2->attach(*downloading, 0, 1, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable2->attach(*shared2, 0, 1, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable2->attach(totalDownloadingSize, 1, 2, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable2->attach(currentDownloadingSize, 2, 3, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);
	dataTable2->attach(downloadingRatio, 3, 4, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	dataTable2->attach(totalSharedSize, 1, 2, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 10);

	speedFrame->add(*dataTable1);
	sizeFrame->add(*dataTable2);

	box1->pack_start(*speedFrame, Gtk::SHRINK, 5);
	box2->pack_start(*sizeFrame, Gtk::SHRINK, 5);

	dataWindow.pack_start(*box1, Gtk::SHRINK, 5);
	dataWindow.pack_start(*box2, Gtk::SHRINK, 5);
}

void DataStatsTab::updateShareSession()
{
	std::ostringstream value;

	if (dsValue){
		value << (usValue/dsValue);
	}
	else
		value << "0.0";

	shareSession.set_label(value.str());
}

void DataStatsTab::updateShareTotal()
{
	std::ostringstream value;

	if (dtValue){
		value << (utValue/dtValue);
	}
	else
		value << "0.0";

	shareTotal.set_label(value.str());
}

void DataStatsTab::updateDownloadingRatio()
{
	std::ostringstream value;

	if (tdsValue)
		value << (cdsValue/tdsValue);
	else
		value << "0.0";

	downloadingRatio.set_label(value.str());
}

DataStatsTab::~DataStatsTab()
{
}
