//           FilesInfoNoteBookTab.cc
//  Mon Sep 10 00:50:16 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA


#include "FilesInfoNoteBookTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include "Lince/Utils.h"

#include <iostream>

using namespace Lince;

FilesInfoNoteBookTab::FilesInfoNoteBookTab():
	nameTab(_("Files")),
	ScrolledWindow()
{
	// Add the TreeView, inside the ScrolledWindow, with the button underneath:
	// Only show the scrollbars when they are necessary:
	add(filesTreeView);
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Create the Tree model:
	refTreeModel = Gtk::ListStore::create(filesColumns);
	filesTreeView.set_model(refTreeModel);
	filesTreeView.set_rules_hint();

	// Add the TreeView's view columns:
	filesTreeView.append_column(_("First"), filesColumns.m_file_firstpiece);
	filesTreeView.append_column(_("Last"), filesColumns.m_file_lastpiece);
	filesTreeView.append_column(_("Name"), filesColumns.m_file_name);
	filesTreeView.append_column(_("Size"), filesColumns.m_file_size);

	Gtk::CellRendererProgress* cell = Gtk::manage(new Gtk::CellRendererProgress);
	int cols_count = filesTreeView.append_column(_("Percentage"), *cell);
	Gtk::TreeViewColumn* pColumn = filesTreeView.get_column(cols_count-1);
	if(pColumn){
#ifdef GLIBMM_PROPERTIES_ENABLED
				pColumn->add_attribute(cell->property_value(), filesColumns.m_file_percentage);
#else
				pColumn->add_attribute(*cell, _("value"), filesColumns.m_file_percentage);
#endif
	}

	filesTreeView.append_column(_("Priority"), filesColumns.m_file_priority);

	for(guint i = 0; i < 6; i++)
	{
		Gtk::TreeView::Column* pColumn = filesTreeView.get_column(i);
		Gtk::CellRenderer* cColumn = filesTreeView.get_column_cell_renderer(i);

		//pColumn->set_reorderable();
		pColumn->set_resizable();

		switch(i)
		{
			case 0:
				pColumn->set_sort_column(filesColumns.m_file_firstpiece);
				break;
			case 1:
				pColumn->set_sort_column(filesColumns.m_file_lastpiece);
				break;
			case 2:
				pColumn->set_sort_column(filesColumns.m_file_name);
				break;
			case 3:
				pColumn->set_sort_column(filesColumns.file_size);
				break;
			case 4:
				pColumn->set_sort_column(filesColumns.m_file_percentage);
				pColumn->set_expand();
				pColumn->set_alignment(0.5);
				cColumn->property_xalign() = 0.0;
				break;
			case 5:
				pColumn->set_sort_column(filesColumns.m_file_priority);
				break;
		}
	}

	filesTreeView.property_can_focus() = false;
	filesTreeView.signal_cursor_changed().connect( sigc::mem_fun(this, &FilesInfoNoteBookTab::unselectRows) );
	filesTreeView.columns_autosize();

	isModelFilled = false;
	downloadedPieces = -1;
}

int FilesInfoNoteBookTab::getFilePercentage(int first, int last, const libtorrent::bitfield *pieces_vector)
{
	int index, count=0;

	last += 1;
	for (index=first; index < last; index++)
	{
		if ((*pieces_vector)[index]) count++;
	}

	return int ((count * 100) / (last - first));
}

void FilesInfoNoteBookTab::updateFilesInfoNoteBookTab(Torrent *t)
{
	int i;
	libtorrent::torrent_info t_info = t->get_torrent_info();
	std::vector<libtorrent::file_entry>::const_iterator file_iterator;
	Gtk::TreeModel::Row row;
	Gtk::TreeModel::iterator row_iter;
	libtorrent::file_entry file_entry;

	if (!isModelFilled)
	{
		isModelFilled = true;

		for (file_iterator = t_info.begin_files(), i = 0; file_iterator != t_info.end_files(); ++file_iterator, i++)
		{
			row = *(refTreeModel->append());
			file_entry = t_info.file_at(i);

			int first = t_info.map_file(i, 0, 1).piece;
			int last = t_info.map_file(i, file_entry.size - 1, 1).piece;

			row[filesColumns.m_file_firstpiece] = first;
			row[filesColumns.m_file_lastpiece] = last;
			row[filesColumns.m_file_name] = file_iterator->path.leaf();
			row[filesColumns.m_file_size] = size_to_string(file_entry.size);
			row[filesColumns.file_size] = file_entry.size;
			row[filesColumns.m_file_percentage] = 0;

			switch (t->get_priority_of_file(i))
			{
				case -1:
					row[filesColumns.m_file_priority] = _("Unknown");
					break;
				case 0:
					row[filesColumns.m_file_priority] = _("None");
					break;
				case 1:
					row[filesColumns.m_file_priority] = _("Normal");
					break;
				case 2:
					row[filesColumns.m_file_priority] = _("High");
					break;
				case 7:
					row[filesColumns.m_file_priority] = _("Maximum");
					break;
			}
		}
	}
	else if (downloadedPieces != t->get_downloaded_pieces())
	{
		downloadedPieces = t->get_downloaded_pieces();
		for (row_iter = refTreeModel->children().begin(), i=0; row_iter != refTreeModel->children().end(); ++row_iter, i++)
		{
			row = *row_iter;

			int first = t_info.map_file(i, 0, 1).piece;
			int last = t_info.map_file(i, row[filesColumns.file_size] - 1, 1).piece;

			if (downloadedPieces != t->get_total_pieces())
			{
				if (t->get_status() != libtorrent::torrent_status::checking_files &&
						t->get_status() != libtorrent::torrent_status::queued_for_checking)
					row[filesColumns.m_file_percentage] = getFilePercentage(first, last, &t->get_pieces());
				else
					row[filesColumns.m_file_percentage] = 0;
			}
			else
				row[filesColumns.m_file_percentage] = 100;
		}
	}
}

void FilesInfoNoteBookTab::unselectRows()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = filesTreeView.get_selection();

	refTreeSelection->unselect_all();
}

FilesInfoNoteBookTab::~FilesInfoNoteBookTab()
{
}
