//           LoadTorrentDialog.cc
//  Thu Nov 22 14:08:15 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"

#include "Utils.h"
#include "LoadTorrentDialog.h"

#include "../config.h"
#include <glibmm/i18n.h>
#include <glib/gstdio.h>

#include "Lince/Utils.h"

#include <fstream>

using namespace Lince;

LoadTorrentDialog::LoadTorrentDialog(int storageMode, const Glib::ustring & _torrentloaddir, const Glib::ustring & _torrentpathsave, std::vector<Utils::torrent_data> * td):
	Dialog(_("Open Torrent"), true, true),
	loadTorrentLabel(_("Torrent File: ")),
	saveDirectoryLabel(_("Save at: "))
{
	torrentpathsave = _torrentpathsave;
	torrentloaddir = _torrentloaddir;
	torrents_data = td;

	saveDirectoryEntry.set_text(torrentpathsave);

	Gtk::Frame *torrentsNamesFrame = Gtk::manage(new Gtk::Frame());
	torrentsNamesFrame->set_shadow_type(Gtk::SHADOW_OUT);
	torrentsNamesFrame->set_label(_("Torrents"));

	Gtk::Frame *torrentFilesFrame = Gtk::manage(new Gtk::Frame());
	torrentFilesFrame->set_shadow_type(Gtk::SHADOW_OUT);
	torrentFilesFrame->set_label(_("Torrents' files"));

	saveDirectorySelectButton.signal_clicked().connect( sigc::mem_fun(*this, &LoadTorrentDialog::selectAndSetSaveDirectory) );
	loadTorrentSelectButton.signal_clicked().connect( sigc::mem_fun(*this, &LoadTorrentDialog::selectAndSetLoadTorrent) );
	loadUrlSelectButton.signal_clicked().connect( sigc::mem_fun(*this, &LoadTorrentDialog::selectAndSetUrl) );
	torrentsFilesTreeView.signal_button_press_event().connect_notify(sigc::mem_fun(*this,  &LoadTorrentDialog::mouse_button_press_event));

	saveDirectorySelectButtonImage.set(Gtk::Stock::DIRECTORY, Gtk::IconSize(1));
	saveDirectorySelectButton.set_image(saveDirectorySelectButtonImage);

	loadTorrentSelectButtonImage.set(Gtk::Stock::DIRECTORY, Gtk::IconSize(1));
	loadTorrentSelectButton.set_image(loadTorrentSelectButtonImage);

	loadUrlSelectButtonImage.set(Gtk::Stock::NETWORK, Gtk::IconSize(1));
	loadUrlSelectButton.set_image(loadUrlSelectButtonImage);

	Gtk::VSeparator * vseparator =  Gtk::manage(new Gtk::VSeparator);
	loadHBox.pack_start(loadTorrentLabel, Gtk::PACK_SHRINK , 5);
	loadHBox.pack_start(loadTorrentEntry, Gtk::PACK_EXPAND_WIDGET, 0);
	loadHBox.pack_start(loadTorrentSelectButton, Gtk::PACK_SHRINK , 5);
	loadHBox.pack_start(*vseparator, Gtk::PACK_SHRINK , 0);
	loadHBox.pack_start(loadUrlSelectButton, Gtk::PACK_SHRINK , 5);

	saveHBox.pack_start(saveDirectoryLabel, Gtk::PACK_SHRINK , 5);
	saveHBox.pack_start(saveDirectoryEntry, Gtk::PACK_EXPAND_WIDGET, 0);
	saveHBox.pack_start(saveDirectorySelectButton, Gtk::PACK_SHRINK , 5);

	torrentsNamesScrolledWindow.add(torrentsNamesTreeView);
	torrentsNamesScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	torrentsFilesScrolledWindow.add(torrentsFilesTreeView);
	torrentsFilesScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Create the Tree model:
	torrentsNamesTreeModel = Gtk::ListStore::create(torrentsNamesColumns);
	torrentsNamesTreeView.set_model(torrentsNamesTreeModel);
	torrentsNamesTreeView.set_rules_hint();
	torrentsNamesTreeSelection = torrentsNamesTreeView.get_selection();

	torrentsFilesTreeModel = Gtk::ListStore::create(torrentsFilesColumns);
	torrentsFilesTreeView.set_model(torrentsFilesTreeModel);
	torrentsFilesTreeView.set_rules_hint();

	// Add the TreeView's view columnsDialogTorrent:
	torrentsNamesTreeView.append_column(_("Name"), torrentsNamesColumns.m_name);

	if (storageMode != 2) torrentsFilesTreeView.append_column_editable(_("Download"), torrentsFilesColumns.m_download);
	else torrentsFilesTreeView.append_column(_("Download"), torrentsFilesColumns.m_download);
	torrentsFilesTreeView.append_column(_("Name"), torrentsFilesColumns.m_name);
	torrentsFilesTreeView.append_column(_("Size"), torrentsFilesColumns.m_size);

	// Priority Column
	Gtk::TreeView::Column *prColumn = Gtk::manage(new Gtk::TreeView::Column(_("Priority")) );
	Gtk::CellRendererCombo *pRenderer = Gtk::manage(new Gtk::CellRendererCombo());

	Utils::ModelColumnsIntString pr_Columns;
	pr_refTreeModel = Gtk::ListStore::create(pr_Columns);

	Gtk::TreeModel::Row row;

	prColumn->pack_start(*pRenderer);
	torrentsFilesTreeView.append_column(*prColumn);

	row = *(pr_refTreeModel->append());
	row[pr_Columns.m_col_id] = 1;
  	row[pr_Columns.m_col_name] = _("Normal");

	row = *(pr_refTreeModel->append());
	row[pr_Columns.m_col_id] = 2;
  	row[pr_Columns.m_col_name] = _("High");

	row = *(pr_refTreeModel->append());
	row[pr_Columns.m_col_id] = 3;
  	row[pr_Columns.m_col_name] = _("Maximum");

	prColumn->add_attribute(pRenderer->property_text(), torrentsFilesColumns.m_prio_choosen);
	pRenderer->property_model() = pr_refTreeModel;
	pRenderer->property_editable() = true;
	pRenderer->property_text_column() = 1;

	pRenderer->signal_edited().connect( sigc::mem_fun(*this, &LoadTorrentDialog::on_cellrenderer_choice_edited) );
	//End priority Column

	for(guint i = 0; i < 3; i++){
		Gtk::TreeView::Column* pColumn = torrentsFilesTreeView.get_column(i);

		if (i == 1)
			pColumn->set_expand();

		pColumn->set_resizable();
	}
	Gtk::CellRendererToggle *toggleRenderer = dynamic_cast<Gtk::CellRendererToggle *>(torrentsFilesTreeView.get_column_cell_renderer(0));
	toggleRenderer->signal_toggled().connect( sigc::mem_fun(*this, &LoadTorrentDialog::downloadFileToggled) );

	m_refActionGroup = Gtk::ActionGroup::create();
	m_refActionGroup->add( Gtk::Action::create("ChangeName", _("Change Name")), sigc::mem_fun(*this, &LoadTorrentDialog::changeName) );

	refUIManager = Gtk::UIManager::create();
	refUIManager->insert_action_group(m_refActionGroup);

	Glib::ustring ui_info =
    "<ui>"
    "  <popup name='PopupMenu'>"
	"    <menuitem action='ChangeName'/>"
    "  </popup>"
    "</ui>";

#ifdef GLIBMM_EXCEPTIONS_ENABLED
	try{
		refUIManager->add_ui_from_string(ui_info);
	}
	catch(const Glib::Error& ex){
		std::cerr << "building menus failed: " <<  ex.what();
	}
#else
	std::auto_ptr<Glib::Error> ex;
	refUIManager->add_ui_from_string(ui_info, ex);
	if(ex.get()){
		std::cerr << "building menus failed: " <<  ex->what();
	}
#endif //GLIBMM_EXCEPTIONS_ENABLED

	pMenuPopup = Gtk::manage(dynamic_cast<Gtk::Menu*>(refUIManager->get_widget("/PopupMenu")));
	if(!pMenuPopup)
		g_warning("menu not found");

	torrentsNamesFrame->add(torrentsNamesScrolledWindow);
	torrentFilesFrame->add(torrentsFilesScrolledWindow);

	Gtk::VBox *vbox = get_vbox();

	vbox->pack_start(loadHBox, Gtk::PACK_SHRINK , 5);
	vbox->pack_start(*torrentsNamesFrame, Gtk::PACK_SHRINK, 5);
	vbox->pack_start(*torrentFilesFrame, Gtk::PACK_EXPAND_WIDGET, 5);
	vbox->pack_start(saveHBox, Gtk::PACK_SHRINK , 5);

	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	okButton = add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

	okButton->signal_clicked().connect( sigc::mem_fun(this, &LoadTorrentDialog::finishDialog) );
	torrentsNamesTreeSelection->signal_changed().connect(sigc::mem_fun(*this, &LoadTorrentDialog::setSelectedTorrentsFiles));

	set_default_size(650, 500);
	set_gravity(Gdk::GRAVITY_CENTER);

	show_all();
	hide();
}

void LoadTorrentDialog::addTorrentsNameRow(const Glib::ustring &name)
{
	Gtk::TreeModel::Row row;

	row = *(torrentsNamesTreeModel->append());

	row[torrentsNamesColumns.m_position] = torrentsNamesTreeModel->children().size() - 1;
	row[torrentsNamesColumns.m_name] = name;
}

void LoadTorrentDialog::addTorrentsFileRow(int position, int priority, const Glib::ustring &name, const libtorrent::size_type &size){
	Gtk::TreeModel::Row row;

	row = *(torrentsFilesTreeModel->append());

	row[torrentsFilesColumns.m_position] = position;

	if (!priority)
		row[torrentsFilesColumns.m_download] = false;
	else
		row[torrentsFilesColumns.m_download] = true;

	row[torrentsFilesColumns.m_name] = name;
	row[torrentsFilesColumns.m_size] = size_to_string((float) size);
	row[torrentsFilesColumns.m_size_type] = size;

	if (priority == 1)
		row[torrentsFilesColumns.m_prio_choosen] = _("Normal");
	else if (priority == 2)
		row[torrentsFilesColumns.m_prio_choosen] = _("High");
	else if (priority == 7)
		row[torrentsFilesColumns.m_prio_choosen] = _("Maximum");

	row[torrentsFilesColumns.m_priority] = pr_refTreeModel;
}

void LoadTorrentDialog::on_cellrenderer_choice_edited(const Glib::ustring& path_string, const Glib::ustring& new_text){
	int torrent_position, file_position;
	Gtk::TreePath path(path_string);

	//Get the row from the path:
	Gtk::TreeModel::iterator iter = torrentsFilesTreeModel->get_iter(path);
	if(iter){
		//Store the user's new text in the model:
		Gtk::TreeRow row = *iter;

		torrent_position = get_selected_torrents_data_position();
		file_position = to_int(path_string.c_str());

		if (!row[torrentsFilesColumns.m_download]) return;
		row[torrentsFilesColumns.m_prio_choosen] = new_text;

		if (row[torrentsFilesColumns.m_prio_choosen] == _("Maximum"))
			(*torrents_data)[torrent_position].priorities[file_position] = 7;

		else if (row[torrentsFilesColumns.m_prio_choosen] == _("High"))
			(*torrents_data)[torrent_position].priorities[file_position] = 2;

		else
			(*torrents_data)[torrent_position].priorities[file_position] = 1;
	}
}

void LoadTorrentDialog::finishDialog(){
	torrentpathsave = saveDirectoryEntry.get_text();
}

void LoadTorrentDialog::selectAndSetSaveDirectory(){
	Glib::ustring directory;

	directory = Utils::getDirectoryFromDialog(_("Select torrent save path"),
											  torrentpathsave);

	if (directory != "")
	{
		saveDirectoryEntry.set_text(directory);
		for (int i=0; i<torrents_data->size(); i++)
			(*torrents_data)[i].savepath = saveDirectoryEntry.get_text();
	}
}

int LoadTorrentDialog::get_selected_torrents_data_position()
{
	Gtk::TreeModel::Row row;
	row = *(torrentsNamesTreeSelection->get_selected());
	return row[torrentsNamesColumns.m_position];
}

void LoadTorrentDialog::setSelectedTorrentsFiles()
{
	int position = 0;
	Utils::torrent_data td;

	torrentsFilesTreeModel->clear();

	td = (*torrents_data)[get_selected_torrents_data_position()];

	for (int i=0; i<td.newfilenames.size(); i++)
	{
		addTorrentsFileRow(position, td.priorities[i], td.newfilenames[i], td.sizes[i]);
		position++;
	}

	saveDirectoryEntry.set_text(td.savepath);
}

void LoadTorrentDialog::addTorrentsDataElement(const Glib::ustring &torrentpath)
{
	libtorrent::torrent_info t(torrentpath.c_str());

	Utils::torrent_data td;

	td.torrentname = t.name();
	td.torrentpath = torrentpath;
	td.savepath = saveDirectoryEntry.get_text();
	td.priorities.resize(t.num_files(), 1);

	for (int i=0; i<t.num_files(); i++){
		td.newfilenames.push_back(t.file_at(i).path.string());
		td.sizes.push_back(t.file_at(i).size);
	}

	torrents_data->push_back(td);
}

void LoadTorrentDialog::selectAndSetLoadTorrent(){
	std::vector<Glib::ustring> torrentsPaths;
	Glib::ustring torrentPath;

	if (loadTorrentEntry.get_text().empty())
		torrentsPaths = Utils::getFilenamesFromLoadDialog(_("Choose a torrent file"), torrentloaddir, _("Torrent files"), "*.torrent");
	else
	{
		if (!fs::exists(loadTorrentEntry.get_text().c_str()))
		{
			Utils::warningDialog(_("Specified file doesn't exists"), loadTorrentEntry.get_text());
			return;
		}

		torrentPath = loadTorrentEntry.get_text();
		torrentsPaths.push_back(torrentPath);
		loadTorrentEntry.set_text("");
	}

	Gtk::TreeModel::iterator row_iter;
	Gtk::TreeModel::Row row;
	for (row_iter = torrentsNamesTreeModel->children().begin(); row_iter != torrentsNamesTreeModel->children().end(); ++row_iter)
	{
		std::vector<Glib::ustring>::iterator names_iter;
		for (names_iter = torrentsPaths.begin(); names_iter != torrentsPaths.end(); ++names_iter)
		{
			row = *row_iter;
			if (row[torrentsNamesColumns.m_name] == *names_iter)
			{
				Utils::warningDialog(_("Specified file is already selected"), *names_iter);
				names_iter = torrentsPaths.erase(names_iter);
				if (names_iter == torrentsPaths.end()) break;
			}
		}
	}

	for(int i=0; i<torrentsPaths.size(); i++)
	{
		try{
			addTorrentsDataElement(torrentsPaths[i]);
			addTorrentsNameRow(torrentsPaths[i]);
			torrentsNamesTreeSelection->select(--(torrentsNamesTreeModel->children().end()));
		}
		catch (libtorrent::invalid_encoding)
		{
			Utils::warningDialog(_("Selected file is not a torrent"), torrentsPaths[i]);
		}
		catch (libtorrent::invalid_torrent_file)
		{
			Utils::warningDialog(_("Selected file is not a torrent"), torrentsPaths[i]);
		}
	}
}

void LoadTorrentDialog::selectAndSetUrl()
{
	std::string name;
	Glib::ustring url = loadTorrentEntry.get_text();

	if (url.empty())
	{
		Utils::warningDialog(_("Not specified any url"), _("Not specified any url"));
		return;
	}

	if (!Utils::isAnURL(url) || !str_has_suffix(url.c_str(), "torrent"))
	{
		Utils::warningDialog(_("Introduced data is not a valid torrent url"), url);
		return;
	}

	int fd = Glib::file_open_tmp(name, "torrent");
	get_http_torrent(url, name);

	Gtk::TreeModel::iterator iter;
	Gtk::TreeModel::Row row;
	for (iter = torrentsNamesTreeModel->children().begin(); iter != torrentsNamesTreeModel->children().end(); ++iter)
	{
		row = *iter;
		if (row[torrentsNamesColumns.m_name] == url)
		{
			Utils::warningDialog(_("Specified url is already selected"), url);
			return;
		}
	}

	try
	{
		addTorrentsDataElement(name);
		addTorrentsNameRow(url);
		torrentsNamesTreeSelection->select(--(torrentsNamesTreeModel->children().end()));
	}
	catch (libtorrent::invalid_encoding)
	{
		Utils::warningDialog(_("Selected file is not a torrent"), url);
	}
	catch (libtorrent::invalid_torrent_file)
	{
		Utils::warningDialog(_("Selected file is not a torrent"), url);
	}

	loadTorrentEntry.set_text("");
	close(fd);
}

void LoadTorrentDialog::downloadFileToggled(const Glib::ustring &s)
{
	int torrent_position, file_position;
	Gtk::TreeModel::Row row;

	torrent_position = get_selected_torrents_data_position();

	row = *(torrentsFilesTreeModel->get_iter(Gtk::TreeModel::Path(s)));

	file_position = row[torrentsFilesColumns.m_position];

	if (row[torrentsFilesColumns.m_download])
	{
		(*torrents_data)[torrent_position].priorities[file_position] = 1;
		row[torrentsFilesColumns.m_prio_choosen] = _("Normal");
	}
	else
	{
		(*torrents_data)[torrent_position].priorities[file_position] = 0;
		row[torrentsFilesColumns.m_prio_choosen] = "";

		bool all_zero = true;
		for (int i=0; i<(*torrents_data)[torrent_position].priorities.size(); i++)
		{
			if ((*torrents_data)[torrent_position].priorities[i] == 1)
			{
				all_zero = false;
				break;
			}
		}
		if (all_zero)
		{
			(*torrents_data)[torrent_position].priorities[file_position] = 1;
			row[torrentsFilesColumns.m_download] = true;
			Utils::warningDialog(_("Selecting files"), _("At least one file of the torrent must be selected"));
		}
	}
}

bool LoadTorrentDialog::isRowSelected(){
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = torrentsFilesTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();

	if(iter){
		return true;
	}

	return false;
}

void LoadTorrentDialog::mouse_button_press_event(GdkEventButton* event){
	if( (event->type == GDK_BUTTON_PRESS) && (event->button == 3) && isRowSelected() ){
		if(pMenuPopup)
			pMenuPopup->popup(event->button, event->time);
	}
}

void LoadTorrentDialog::changeName(){
	int torrent_position, file_position;
	Glib::ustring new_name;
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = torrentsFilesTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
	Gtk::TreeModel::Row row = *iter;

	if (!row[torrentsFilesColumns.m_download]) return;

	torrent_position = get_selected_torrents_data_position();
	file_position = row[torrentsFilesColumns.m_position];

	new_name = Utils::askNameDialog(_("Choose a new name"), row[torrentsFilesColumns.m_name]);
	if(new_name.empty()) return;

	if (row[torrentsFilesColumns.m_name] != new_name)
	{
		row[torrentsFilesColumns.m_name] = new_name;

/*#if defined(_WIN32) && defined(__MINGW32__)
		if ((*torrents_data)[torrent_position].newfilenames.size() > 1)
			new_name = (*torrents_data)[torrent_position].torrentname + "\\" +
					new_name;
#else
		if ((*torrents_data)[torrent_position].newfilenames.size() > 1)
			new_name = (*torrents_data)[torrent_position].torrentname + "/" +
					new_name;
#endif

		(*torrents_data)[torrent_position].newmap[file_position].path =
			boost::filesystem::path(new_name.c_str());*/

		(*torrents_data)[torrent_position].newfilenames[file_position] = new_name;

	}
}

LoadTorrentDialog::~LoadTorrentDialog(){
}
