//           NoteBook.cc
//  Sat Feb 17 22:10:50 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "NoteBookManager.h"
#include "MenuBar.h"

#include "Lince/Core.h"
#include "Lince/Utils.h"

#include <iostream>
#include <algorithm>

using namespace Lince;

NoteBookManager::NoteBookManager(MainWindow *m, MenuBar *mb, Notifications *nf):
	Gtk::Notebook(),
	TT(m, mb),
	OT(this),
	ST()
{
	set_name("mainNotebook");
	set_border_width(0);
    set_show_tabs(true);
	set_scrollable(true);
	set_show_border(true);

    set_tab_pos(Gtk::POS_TOP);

	set_border_width(0);
	property_tab_hborder() = 0;
	property_tab_vborder() = 0;
	property_can_focus() = true;

	menuBar = mb;
	notifications = nf;

	last_page = -1;
	page_positions.reserve(LAST_TAB);

	for (int i=OPTIONS_TAB; i<LAST_TAB; i++){
		page_positions[i] = -1;
	}

	signal_switch_page().connect( sigc::mem_fun(*this, &NoteBookManager::on_my_switch_page) );
}

NoteBookManager::~NoteBookManager(){
}

void NoteBookManager::addTorrentsTab(){
	if (!TT.isTabShown()){
		last_page++;
		TT.setShownTab(true);
		/*page_positions[TORRENTS_TAB] = last_page;
		append_page(*TT.getTabChild(), *TT.getTabWidget());*/

		page_positions[TORRENTS_TAB] = 0;
		insert_page(*TT.getTabChild(), *TT.getTabWidget(), 0);
		for (int i=OPTIONS_TAB; i<LAST_TAB; i++){
			if (page_positions[i] != -1){
				page_positions[i] += 1;
			}
		}

		set_menu_label_text(*TT.getTabChild(), "Torrents");
		TT.getTabButton()->signal_clicked().connect( sigc::mem_fun(*this, &NoteBookManager::removeTorrentsTab) );
		show_all();
	}

	set_current_page(page_positions[TORRENTS_TAB]);
}

void NoteBookManager::removeTorrentsTab(void){
	if (page_positions[TORRENTS_TAB] != -1){
		updatePagePositions(page_positions[TORRENTS_TAB]);
		last_page--;
		page_positions[TORRENTS_TAB] = -1;
		TT.setShownTab(false);
		remove_page(*TT.getTabChild());
	}
}

void NoteBookManager::addOptionsTab(){
	if (!OT.isTabShown()){
		last_page++;
		OT.setShownTab(true);
		/*page_positions[OPTIONS_TAB] = last_page;
		append_page(*OT.getTabChild(), *OT.getTabWidget());*/

		int position = 0;
		if (page_positions[TORRENTS_TAB] != -1){
			position += 1;
		}

		page_positions[OPTIONS_TAB] = position;
		insert_page(*OT.getTabChild(), *OT.getTabWidget(), position);

		for (int i=STATISTICS_TAB; i<LAST_TAB; i++){
			if (page_positions[i] != -1){
				page_positions[i] += 1;
			}
		}
		set_menu_label_text(*OT.getTabChild(), "Options");
		OT.getTabButton()->signal_clicked().connect( sigc::mem_fun(*this, &NoteBookManager::removeOptionsTab) );
		show_all();
	}

	set_current_page(page_positions[OPTIONS_TAB]);
}

void NoteBookManager::removeOptionsTab(void){
	if (page_positions[OPTIONS_TAB] != -1){
		updatePagePositions(page_positions[OPTIONS_TAB]);
		last_page--;
		page_positions[OPTIONS_TAB] = -1;
		OT.setShownTab(false);
		remove_page(*OT.getTabChild());
	}
}

void NoteBookManager::addStatisticsTab(void){
	if (!ST.isTabShown()){
		last_page++;
		ST.setShownTab(true);
		/*page_positions[STATISTICS_TAB] = last_page;
		append_page(*ST.getTabChild(), *ST.getTabWidget());*/

		int position = 0;
		if (page_positions[TORRENTS_TAB] != -1){
			position += 1;
		}
		if (page_positions[OPTIONS_TAB] != -1){
			position += 1;
		}
		page_positions[STATISTICS_TAB] = position;
		insert_page(*ST.getTabChild(), *ST.getTabWidget(), position);

		for (int i=LOG_TAB; i<LAST_TAB; i++){
			if (page_positions[i] != -1){
				page_positions[i] += 1;
			}
		}

		set_menu_label_text(*ST.getTabChild(), "Statistics");
		ST.getTabButton()->signal_clicked().connect( sigc::mem_fun(*this, &NoteBookManager::removeStatisticsTab) );
		show_all();
	}

	set_current_page(page_positions[STATISTICS_TAB]);
}

void NoteBookManager::removeStatisticsTab(void){
	if (page_positions[STATISTICS_TAB] != -1){
		updatePagePositions(page_positions[STATISTICS_TAB]);
		last_page--;
		page_positions[STATISTICS_TAB] = -1;
		ST.setShownTab(false);
		remove_page(*ST.getTabChild());
	}
}

void NoteBookManager::addLogTab(void){
	if (!LT.isTabShown()){
		last_page++;
		LT.setShownTab(true);
		/*page_positions[LOG_TAB] = last_page;
		append_page(*LT.getTabChild(), *LT.getTabWidget());*/

		int position = 0;
		if (page_positions[TORRENTS_TAB] != -1){
			position += 1;
		}
		if (page_positions[OPTIONS_TAB] != -1){
			position += 1;
		}
		if (page_positions[STATISTICS_TAB] != -1){
			position += 1;
		}

		page_positions[LOG_TAB] = position;
		insert_page(*LT.getTabChild(), *LT.getTabWidget(), position);

		if (page_positions[SEARCH_TAB] != -1) page_positions[SEARCH_TAB] += 1;

		set_menu_label_text(*LT.getTabChild(), "Log");
		LT.getTabButton()->signal_clicked().connect( sigc::mem_fun(*this, &NoteBookManager::removeLogTab) );
		show_all();
	}

	set_current_page(page_positions[LOG_TAB]);
}

void NoteBookManager::removeLogTab(void){
	if (page_positions[LOG_TAB] != -1){
		updatePagePositions(page_positions[LOG_TAB]);
		last_page--;
		page_positions[LOG_TAB] = -1;
		LT.setShownTab(false);
		remove_page(*LT.getTabChild());
	}
}

void NoteBookManager::addSearchTab(void){
	if (!SChT.isTabShown())
	{
		last_page++;
		SChT.setShownTab(true);

		int position = 0;
		if (page_positions[TORRENTS_TAB] != -1){
			position += 1;
		}
		if (page_positions[OPTIONS_TAB] != -1){
			position += 1;
		}
		if (page_positions[STATISTICS_TAB] != -1){
			position += 1;
		}
		if (page_positions[LOG_TAB] != -1){
			position += 1;
		}

		page_positions[SEARCH_TAB] = position;
		insert_page(*SChT.getTabChild(), *SChT.getTabWidget(), position);

		set_menu_label_text(*SChT.getTabChild(), "Search");
		SChT.getTabButton()->signal_clicked().connect( sigc::mem_fun(*this, &NoteBookManager::removeSearchTab) );
		show_all();
	}

	set_current_page(page_positions[SEARCH_TAB]);
}

void NoteBookManager::removeSearchTab(void){
	if (page_positions[SEARCH_TAB] != -1){
		updatePagePositions(page_positions[SEARCH_TAB]);
		last_page--;
		page_positions[SEARCH_TAB] = -1;
		SChT.setShownTab(false);
		remove_page(*SChT.getTabChild());
	}
}

bool NoteBookManager::isInfoTabListed(const libtorrent::sha1_hash &sh, TorrentsInfoTab::iterator &i){
	i =	std::find(TIT.begin(), TIT.end(), sh);
	return (i != TIT.end());
}

void NoteBookManager::addTorrentInfoTab(Lince::Torrent *t){
	bool isListed;
	TorrentsInfoTab::iterator i;

	isListed = isInfoTabListed(t->get_hash(), i);

	if (!isListed){
		Glib::ustring torrentName(t->name());
		class TorrentInfoTab IT(torrentName, t->get_hash(), this);
		TIT.push_back(IT);

		i=--(TIT.end());
		last_page++;
		i->setShownTab(true);
		append_page(*i->getTabChild(), *i->getTabWidget());
		set_menu_label_text(*i->getTabChild(), torrentName);
		i->getTabButton()->signal_clicked().connect( sigc::bind<libtorrent::sha1_hash> (sigc::mem_fun(*this, &NoteBookManager::removeInfoTab), t->get_hash()) );
		show_all();
		i->updateTorrentInfoTab(t);
		set_current_page(last_page);
	}
	else{
		set_current_page(page_num(*i->getTabChild()));
	}
}

void NoteBookManager::removeInfoTab(const libtorrent::sha1_hash &sh){
	TorrentsInfoTab::iterator i;

	i = std::find(TIT.begin(), TIT.end(), sh);

	if (i != TIT.end()){
		updatePagePositions(page_num(*i->getTabChild()));
		last_page--;
		i->setShownTab(false);
		remove_page(*i->getTabChild());
		TIT.erase(i);
	}
}

void NoteBookManager::updateTorrentsInfoTab()
{
	int i;
	TorrentsInfoTab::iterator j;

	TorrentsManagerPtr tm;
	tm = Core::get_torrents_manager();
	Torrent *t;

	for (i = 1; i < tm->downloading_torrents_count() + 1; i++)
	{
		t = tm->get_downloading_torrent_at(i);

		// Puede que haya que usar el nombre de la estructura info
		j = std::find(TIT.begin(), TIT.end(), t->get_hash());
		if (j != TIT.end())
		{
			j->updateTorrentInfoTab(t);
		}
	}

	for (i = 1; i < tm->shared_torrents_count() + 1; i++)
	{
		t = tm->get_shared_torrent_at(i);

		// Puede que haya que usar el nombre de la estructura info
		j = std::find(TIT.begin(), TIT.end(), t->get_hash());
		if (j != TIT.end())
		{
			j->updateTorrentInfoTab(t);
		}
	}
}

void NoteBookManager::updatePagePositions(int value){
	for (int i=FIRST_TAB; i!=LAST_TAB; i++){
		if (page_positions[i] > value){
			page_positions[i] -= 1;
		}
	}
}

void NoteBookManager::getSelectedTabName(Glib::ustring &tabName){
	int position = get_current_page();
	if (position >= 0){
		tabName = get_current()->get_menu_label_text();
	}
	else{
		tabName = "";
	}
}

int NoteBookManager::getActivePage(){
	return get_current_page();
}

void NoteBookManager::on_my_switch_page(GtkNotebookPage* page, guint pageNum){
	if ( isTorrentsInfoTabSelected() )
		menuBar->setTorrentTabSelected();
	else if (isTorrentsTabSelected() && (TT.getDownloadingSelectedTorrents() == 1 || TT.getDownloadedSelectedTorrents() == 1 ))
		menuBar->setTorrentSelected();
	else if (isTorrentsTabSelected() && (TT.getDownloadingSelectedTorrents() > 1 || TT.getDownloadedSelectedTorrents() > 1 ))
		menuBar->setTorrentsSelected();
	else
		menuBar->setTorrentSelected(false);
}

bool NoteBookManager::isTorrentsTabSelected(){
	int position = get_current_page();

	if (position == page_positions[TORRENTS_TAB])
		return true;

	return false;
}

bool NoteBookManager::isTorrentsInfoTabSelected(){
	int position = get_current_page();

	if (position != page_positions[TORRENTS_TAB] && position != page_positions[OPTIONS_TAB] &&
		position != page_positions[STATISTICS_TAB] && position != page_positions[LOG_TAB] &&
		position != page_positions[SEARCH_TAB]){
			return true;
		}

	return false;
}

void NoteBookManager::removeCurrentTab(){
	int position = get_current_page();

	if (position == page_positions[TORRENTS_TAB]){
		removeTorrentsTab();
	}
	else if (position == page_positions[OPTIONS_TAB]){
		removeOptionsTab();
	}
	else if (position == page_positions[STATISTICS_TAB]){
		removeStatisticsTab();
	}
	else if (position == page_positions[LOG_TAB]){
		removeLogTab();
	}
	else{
		TorrentsInfoTab::iterator i;
		Glib::ustring name = get_current()->get_menu_label_text();
		i = std::find(TIT.begin(), TIT.end(), name);
		if (i != TIT.end())
			removeInfoTab(i->getTorrentHash());
	}
}

bool NoteBookManager::isTorrentInfoTabShown(const libtorrent::sha1_hash & th){
	TorrentsInfoTab::iterator i;

	return isInfoTabListed(th, i);
}

torrentPosition NoteBookManager::getSelectedTorrentInfoTabPosition(void)
{
	TorrentsInfoTab::iterator i;
	Glib::ustring name = get_current()->get_menu_label_text();

	i = std::find(TIT.begin(), TIT.end(), name);
	if (i != TIT.end())
		return Core::get_torrents_manager()->get_torrent_position(i->getTorrentHash());

	return std::make_pair(0, false);
}

void NoteBookManager::banClient(Glib::ustring &ban_data, int ban_time)
{
	SessionManagerPtr ssm;
	ssm = Core::get_session_manager();
	ssm->get_ip_filter()->add_ipv4_rule(ban_data, 1);
	ssm->update_ip_filter();

	if (ban_time != -1)
		Glib::signal_timeout().connect( sigc::bind<Glib::ustring>(sigc::mem_fun
			(*this, &NoteBookManager::unbanClient), ban_data), (ban_time * 1000));

	OT.updateIpFilterData();
}

bool NoteBookManager::unbanClient(Glib::ustring &ban_data){
	SessionManagerPtr ssm;

	ssm = Core::get_session_manager();
	ssm->get_ip_filter()->add_ipv4_rule(ban_data, 128);
	ssm->update_ip_filter();

	OT.updateIpFilterData();
	return false;
}
