/***************************************************************************
 *            SearchTab.cc
 *
 *  Mon Sep 22 16:56:32 2008
 *  Copyright  2008  Fernando Tarín Morales
 *  <icemanf@gmail.com>
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "SearchTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include <gtkmm/button.h>
#include <gtkmm/label.h>

#include <sys/types.h>
#include <sys/wait.h>

using namespace Lince;

SearchTab::SearchTab():
	BaseTab(_("Search"))
{
	Gtk::Label *search_label = Gtk::manage(new Gtk::Label(_("Search:")));
	Gtk::HBox *search_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Button *search_button = Gtk::manage(new Gtk::Button(_("Search")));
	Gtk::Button *stop_button = Gtk::manage(new Gtk::Button(_("Stop")));
		
	// Signals
	search_data.signal_activate().connect( sigc::mem_fun(*this, &SearchTab::startSearch) );
	search_button->signal_clicked().connect( sigc::mem_fun(*this, &SearchTab::startSearch) );
	stop_button->signal_clicked().connect( sigc::mem_fun(*this, &SearchTab::stopSearch) );
		
	// Fill containers
	search_row->pack_start(*search_label, Gtk::PACK_SHRINK, 5);
	search_row->pack_start(search_data, Gtk::PACK_EXPAND_WIDGET, 5);
	search_row->pack_start(*search_button, Gtk::PACK_SHRINK, 5);
	search_row->pack_start(*stop_button, Gtk::PACK_SHRINK, 5);
	
	vbox.pack_start(*search_row, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(searchs_notebook, Gtk::PACK_EXPAND_WIDGET, 5);
}

void SearchTab::startSearch()
{
	if (search_data.get_text().empty())
		return;
	
	SearchTabResults results_tab(gsm, search_data.get_text());
	results_list.push_back(results_tab);
	
	int current_page;
	std::list< SearchTabResults >::iterator i;
	
	i = --(results_list.end());
	i->setShownTab(true);
	current_page = searchs_notebook.append_page(*i->getTabChild(), *i->getTabWidget());
	i->getTabButton()->signal_clicked().connect( sigc::bind<const Gtk::Widget *> (sigc::mem_fun(*this, &SearchTab::removeSearchTab), i->getTabChild()) );
	i->startSearch();
	searchs_notebook.show_all();
	searchs_notebook.set_current_page(current_page);
}

void SearchTab::stopSearch()
{
	int current_page = searchs_notebook.get_current_page();
	std::list< SearchTabResults >::iterator iter = results_list.begin();
	
	for (int i = 0; i < current_page; i++)
		iter++;
		
	iter->stopSearch();
}

void SearchTab::removeSearchTab(const Gtk::Widget *w)
{
	std::list< SearchTabResults >::iterator iter = results_list.begin();
	int position = searchs_notebook.page_num(*w);
	
	for (int i = 0; i < position; i++)
		iter++;
		
	if (iter->isRunning())
	{
		iter->stopSearch();
		wait(NULL);
	}
		
	searchs_notebook.remove_page(*iter->getTabChild());
	results_list.erase(iter);
}

SearchTab::~SearchTab()
{
}
