/***************************************************************************
 *            SearchTabResults.h
 *
 *  Sat Oct  4 13:37:44 2008
 *  Copyright  2008  Fernando Tarín Morales
 *  <icemanf@gmail.com>
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#ifndef _SEARCHTABRESULTS_H
#define _SEARCHTABRESULTS_H

#include <gtkmm/label.h>
#include <gtkmm/box.h>
#include <gtkmm/treeview.h>
#include <gtkmm/treemodel.h>

#include "Lince/Utils.h"

#include "BaseTab.h"
#include "GuiSetupManager.h"

class SearchResults: public Gtk::VBox
{
	private:
		Glib::ustring search_terms;
		Gtk::Label status, results;
		
		class searchResultsModelColumns: public Gtk::TreeModel::ColumnRecord{
		public:
			searchResultsModelColumns(){
				add(m_position); add(m_link); add(m_name); add(m_size);
				add(m_seeds); add(m_leechs); add(m_engine); add(size);
			}
			Gtk::TreeModelColumn< int > m_position;
			Gtk::TreeModelColumn< Glib::ustring > m_link;
			Gtk::TreeModelColumn< Glib::ustring > m_name;
			Gtk::TreeModelColumn< Glib::ustring > m_size;
			Gtk::TreeModelColumn< int > m_seeds;
			Gtk::TreeModelColumn< int > m_leechs;
			Gtk::TreeModelColumn< Glib::ustring > m_engine;
			
			Gtk::TreeModelColumn< boost::int64_t > size;
		};
		
		Gtk::TreeView searchResultsTreeView;
		Glib::RefPtr<Gtk::ListStore> searchResultsTreeModel;
		searchResultsModelColumns searchResultsColumns;
		
		GuiSetupManager * gsm;
	
		// Search
		Glib::Pid pid;
		bool is_running;
		Glib::RefPtr<Glib::IOChannel> io_stdout, io_stderr;
		
		void addRow(const Glib::ustring&, const Glib::ustring&, boost::int64_t,
					int, int, const Glib::ustring&);
		
		void searchFinished(int, int);
		bool showResults();
		void row_activated(const Gtk::TreeModel::Path&, Gtk::TreeViewColumn*);
		void downloadTorrent();
	
	public:
		SearchResults(GuiSetupManager *, const Glib::ustring &_search_terms);
		~SearchResults();
		
		void startSearch();
		void stopSearch();
		bool isRunning() { return is_running; };
};

class SearchTabResults: public BaseTab
{
	private:
		GuiSetupManager * gsm;
		Glib::ustring search_terms;
		SearchResults search_results;
		
	public:
		SearchTabResults(GuiSetupManager *, const Glib::ustring &);
		SearchTabResults(const SearchTabResults& t);
		~SearchTabResults();
		
		void startSearch() { search_results.startSearch(); };
		void stopSearch() { search_results.stopSearch(); };
		bool isRunning() { return search_results.isRunning(); };
		
		virtual Gtk::Widget * getTabChild(void) { return & search_results; };
};

#endif /* SEARCHTABRESULTS_H */ 
