//           StatusBar.cc
//  Tue Feb 27 23:57:46 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "Utils.h"
#include "StatusBar.h"

#include "Lince/Utils.h"

#include "../config.h"
#include <glibmm/i18n.h>
#include <glibmm/ustring.h>

using namespace Lince;

StatusBar::StatusBar():
	Gtk::Statusbar(),
	statusGoodLabel(_("Good")),
	statusFirewalledLabel(_("Firewalled")),
	statusNoConnectionLabel(_("No connection")),
	statusSeedingLabel(_("Seeding")),
	upLabel(_("Up: ")),
	downLabel(_("Down: ")),
	connectionsLabel(_("Connections: ")),
	dhTLabel("DHT:"),
	dhtStatusLabel(_("Disabled"))
{

	Glib::RefPtr<Gtk::IconFactory> statusfactory = Gtk::IconFactory::create();
	Utils::registerIcon(statusfactory, "connection-ok.png", "connection-ok", "connection ok");
	Utils::registerIcon(statusfactory, "no-connection.png", "no-connection", "no connection");
	Utils::registerIcon(statusfactory, "firewalled.png", "firewalled", "firewalled");
	Utils::registerIcon(statusfactory, "seeding.png", "seeding", "seeding");
	statusfactory->add_default();

	upImage.set(Gtk::Stock::GO_UP, Gtk::IconSize(1));
	downImage.set(Gtk::Stock::GO_DOWN, Gtk::IconSize(1));
	statusGoodImage.set(Gtk::StockID("connection-ok"), Gtk::IconSize(1));
	statusNoConnectionImage.set(Gtk::StockID("no-connection"), Gtk::IconSize(1));
	statusFirewalledImage.set(Gtk::StockID("firewalled"), Gtk::IconSize(1));
	statusSeedingImage.set(Gtk::StockID("seeding"), Gtk::IconSize(1));
	connectionsImage.set(Gtk::Stock::NETWORK, Gtk::IconSize(1));

	Gtk::HBox *hboxA, *hbox1, *hbox2, *hbox3, *hbox4;

	statusHBox.pack_start(statusNoConnectionImage, Gtk::PACK_SHRINK , 0);
	statusHBox.pack_start(statusNoConnectionLabel, Gtk::PACK_SHRINK , 5);
	statusHBox.pack_start(statusGoodImage, Gtk::PACK_SHRINK , 0);
	statusHBox.pack_start(statusGoodLabel, Gtk::PACK_SHRINK , 5);
	statusHBox.pack_start(statusFirewalledImage, Gtk::PACK_SHRINK , 0);
	statusHBox.pack_start(statusFirewalledLabel, Gtk::PACK_SHRINK , 5);
	statusHBox.pack_start(statusSeedingImage, Gtk::PACK_SHRINK , 0);
	statusHBox.pack_start(statusSeedingLabel, Gtk::PACK_SHRINK , 5);

	hbox1 = Gtk::manage(new Gtk::HBox);
	hbox1->pack_start(downImage, Gtk::PACK_SHRINK , 0);
	hbox1->pack_start(downLabel, Gtk::PACK_SHRINK , 5);
	hbox1->pack_start(maxDownValueLabel, Gtk::PACK_SHRINK , 0);
	hbox1->pack_start(downValueLabel, Gtk::PACK_SHRINK , 5);

	hbox2 = Gtk::manage(new Gtk::HBox);
	hbox2->pack_start(upImage, Gtk::PACK_SHRINK , 0);
	hbox2->pack_start(upLabel, Gtk::PACK_SHRINK , 5);
	hbox2->pack_start(maxUpValueLabel, Gtk::PACK_SHRINK , 0);
	hbox2->pack_start(upValueLabel, Gtk::PACK_SHRINK , 5);

	hbox3 = Gtk::manage(new Gtk::HBox);
	hbox3->pack_start(connectionsImage, Gtk::PACK_SHRINK , 0);
	hbox3->pack_start(connectionsLabel, Gtk::PACK_SHRINK , 5);
	hbox3->pack_start(maxConnectsValue, Gtk::PACK_SHRINK , 0);
	hbox3->pack_start(connectionsValue, Gtk::PACK_SHRINK , 5);

	hbox4 = Gtk::manage(new Gtk::HBox);
	hbox4->pack_start(dhTLabel, Gtk::PACK_SHRINK , 5);
	hbox4->pack_start(dhtStatusLabel, Gtk::PACK_SHRINK , 0);

	maxUpValueLabel.hide();
	maxDownValueLabel.hide();
	maxConnectsValue.hide();

	hboxA = Gtk::manage(new Gtk::HBox);
	hboxA->pack_end(*hbox2, Gtk::PACK_SHRINK , 10);
	hboxA->pack_end(*hbox1, Gtk::PACK_SHRINK , 10);
	hboxA->pack_end(*hbox3, Gtk::PACK_SHRINK , 10);
	hboxA->pack_end(*hbox4, Gtk::PACK_SHRINK , 15);
	hboxA->pack_end(statusHBox, Gtk::PACK_SHRINK , 25);

	statusHBox.hide();

	pack_start(*hboxA);
}

void StatusBar::setUpSpeed(float value){
	upValueLabel.set_text(size_to_string(value));
}

void StatusBar::setDownSpeed(float value){
	downValueLabel.set_text(size_to_string(value));
}

void StatusBar::setMaxUpSpeed(float value){
	if (value == -1) maxUpValueLabel.hide();
	else{
		maxUpValueLabel.set_text(std::string("[") + size_to_string(value) + std::string("]"));
		maxUpValueLabel.show();
	}
}

void StatusBar::setMaxDownSpeed(float value){
	if (value == -1) maxDownValueLabel.hide();
	else{
		maxDownValueLabel.set_text(std::string("[") + size_to_string(value) + std::string("]"));
		maxDownValueLabel.show();
	}
}

void StatusBar::setMaxConnections(int value){
	if (!value) maxConnectsValue.hide();
	else{
		maxConnectsValue.set_text(Glib::ustring::compose("[%1]", value));
		maxConnectsValue.show();
	}
}

void StatusBar::setStatus(status s){
	statusHBox.children().erase(statusHBox.children().begin(), statusHBox.children().end());

	if (s == GOOD){
		statusHBox.pack_start(statusGoodImage, Gtk::PACK_SHRINK , 0);
		statusHBox.pack_start(statusGoodLabel, Gtk::PACK_SHRINK , 5);
	}
	else if (s == FIREWALLED){
		statusHBox.pack_start(statusFirewalledImage, Gtk::PACK_SHRINK , 0);
		statusHBox.pack_start(statusFirewalledLabel, Gtk::PACK_SHRINK , 5);
	}
	else if (s == NO_CONNECTION){
		statusHBox.pack_start(statusNoConnectionImage, Gtk::PACK_SHRINK , 0);
		statusHBox.pack_start(statusNoConnectionLabel, Gtk::PACK_SHRINK , 5);
	}
	else{
		statusHBox.pack_start(statusSeedingImage, Gtk::PACK_SHRINK , 0);
		statusHBox.pack_start(statusSeedingLabel, Gtk::PACK_SHRINK , 5);
	}

	statusHBox.show();
}

void StatusBar::setConnectionsNum(int value){
	connectionsValue.set_text(Glib::ustring::compose("%1", value));
}

void StatusBar::setDhtStatus(bool value, int dht_nodes, int dht_global_nodes){
	std::ostringstream dht;
	if (value)
	{
		dht << dht_nodes << ":" << dht_global_nodes;
		dhtStatusLabel.set_label(dht.str());
	}
	else
		dhtStatusLabel.set_label(_("Disabled"));
}

StatusBar::~StatusBar(){
}
