//           SystemTray.cc
//  Mon Dec 24 15:49:42 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "SystemTray.h"
#include "MainWindow.h"
#include "Utils.h"

#include "Lince/Core.h"
#include "Lince/Utils.h"
#include "Lince/SessionManager.h"
#include "Lince/TorrentsManager.h"

#include "../config.h"
#include <glibmm/i18n.h>

using namespace Lince;

SystemTray::SystemTray(MainWindow *m){
	Glib::ustring logoFile = Glib::build_filename(ICON_DIR, "lince.png");
	
	// Icon
	systemTray = Gtk::StatusIcon::create(Gdk::Pixbuf::create_from_file(logoFile));
	shown = false;
	
	// Signals
	systemTray->signal_activate().connect(sigc::mem_fun(*this, &SystemTray::toggleShowButton));
	systemTray->signal_popup_menu().connect(sigc::mem_fun(*this,  &SystemTray::on_my_popup_menu));
		
	// Menu actions
	m_refActionGroup = Gtk::ActionGroup::create();
	m_refActionGroup->add( Gtk::Action::create("SystrayMenu", "Systray Menu"));
	showLince = Gtk::ToggleAction::create("ShowLince", _("_Show Lince"), "Show Lince");
	showLince->set_active();
	m_refActionGroup->add( showLince, sigc::mem_fun(m, &MainWindow::manage_window) );
	
	m_refActionGroup->add( Gtk::Action::create("Upload", _("Upload")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedus", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), -1) );
	m_refActionGroup->add( Gtk::Action::create("10us", "10 KB", "10 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 10) );
	m_refActionGroup->add( Gtk::Action::create("20us", "20 KB", "20 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 20) );
	m_refActionGroup->add( Gtk::Action::create("40us", "40 KB", "40 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 40) );
	m_refActionGroup->add( Gtk::Action::create("60us", "60 KB", "60 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 60) );
	m_refActionGroup->add( Gtk::Action::create("80us", "80 KB", "80 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 80) );
	m_refActionGroup->add( Gtk::Action::create("100us", "100 KB", "100 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 100) );
	m_refActionGroup->add( Gtk::Action::create("200us", "200 KB", "200 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 200) );
	m_refActionGroup->add( Gtk::Action::create("400us", "400 KB", "400 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 400) );
	m_refActionGroup->add( Gtk::Action::create("Manualus", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_limit_from_systray), 0) );
	
	
	m_refActionGroup->add( Gtk::Action::create("Download", _("Download")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedds", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), -1) );
	m_refActionGroup->add( Gtk::Action::create("10ds", "10 KB", "10 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 10) );
	m_refActionGroup->add( Gtk::Action::create("20ds", "20 KB", "20 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 20) );
	m_refActionGroup->add( Gtk::Action::create("40ds", "40 KB", "40 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 40) );
	m_refActionGroup->add( Gtk::Action::create("60ds", "60 KB", "60 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 60) );
	m_refActionGroup->add( Gtk::Action::create("80ds", "80 KB", "80 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 80) );
	m_refActionGroup->add( Gtk::Action::create("100ds", "100 KB", "100 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 100) );
	m_refActionGroup->add( Gtk::Action::create("200ds", "200 KB", "200 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 200) );
	m_refActionGroup->add( Gtk::Action::create("400ds", "400 KB", "400 KB"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 400) );
	m_refActionGroup->add( Gtk::Action::create("Manualds", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_download_limit_from_systray), 0) );
	
	m_refActionGroup->add( Gtk::Action::create("Connections", _("Max Connections")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedc", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), -1) );
	m_refActionGroup->add( Gtk::Action::create("10c", "10", "10"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 10) );
	m_refActionGroup->add( Gtk::Action::create("20c", "20", "20"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 20) );
	m_refActionGroup->add( Gtk::Action::create("40c", "40", "40"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 40) );
	m_refActionGroup->add( Gtk::Action::create("60c", "60", "60"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 60) );
	m_refActionGroup->add( Gtk::Action::create("80c", "80", "80"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 80) );
	m_refActionGroup->add( Gtk::Action::create("100c", "100", "100"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 100) );
	m_refActionGroup->add( Gtk::Action::create("200c", "200", "200"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 200) );
	m_refActionGroup->add( Gtk::Action::create("400c", "400", "400"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 400) );
	m_refActionGroup->add( Gtk::Action::create("Manualc", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_connections_limit_from_systray), 0) );	
	
	m_refActionGroup->add( Gtk::Action::create("UploadSlots", _("Upload Slots")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedst", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), -1) );
	m_refActionGroup->add( Gtk::Action::create("5st", "5", "5"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 5) );
	m_refActionGroup->add( Gtk::Action::create("10st", "10", "10"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 10) );
	m_refActionGroup->add( Gtk::Action::create("15st", "15", "15"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 15) );
	m_refActionGroup->add( Gtk::Action::create("25st", "25", "25"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 25) );
	m_refActionGroup->add( Gtk::Action::create("35st", "35", "35"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 35) );
	m_refActionGroup->add( Gtk::Action::create("50st", "50", "50"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 50) );
	m_refActionGroup->add( Gtk::Action::create("75st", "75", "75"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 75) );
	m_refActionGroup->add( Gtk::Action::create("100st", "100", "100"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 100) );
	m_refActionGroup->add( Gtk::Action::create("Manualst", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(m, &MainWindow::set_upload_slots_limit_from_systray), 0) );	
	
	m_refActionGroup->add( Gtk::Action::create("ResumeTorrents", _("Resume Torrents"), "Resume Torrents"), sigc::mem_fun(m, &MainWindow::start_all_torrents) );
	m_refActionGroup->add( Gtk::Action::create("StopTorrents", _("Stop Torrents"), "Stop Torrents"), sigc::mem_fun(m, &MainWindow::stop_all_torrents) );
	
	// TODO: Try to make this local to systray
	m_refActionGroup->add( Gtk::Action::create("Quit", Gtk::Stock::QUIT, _("_Quit"), "_Quit"), sigc::mem_fun(m, &MainWindow::signal_exit) );
	
	s_refUIManager = Gtk::UIManager::create();
	s_refUIManager->insert_action_group(m_refActionGroup);
	
	Glib::ustring s_ui_info =
    "<ui>"
    "  <popup name='PopupMenu'>"
    "    <menuitem action='ShowLince'/>"
	"    <separator/>"
	"	 <menu action='Upload'>"
	"    <menuitem action='Unlimitedus'/>"
	"    <separator/>"
	"    <menuitem action='10us'/>"
	"    <menuitem action='20us'/>"
	"    <menuitem action='40us'/>"
	"    <menuitem action='60us'/>"
	"    <menuitem action='80us'/>"
	"    <menuitem action='100us'/>"
	"    <menuitem action='200us'/>"
	"    <menuitem action='400us'/>"
	"    <separator/>"
	"    <menuitem action='Manualus'/>"
	"	 </menu>"		
	"	 <menu action='Download'>"
	"    <menuitem action='Unlimitedds'/>"
	"    <separator/>"
	"    <menuitem action='10ds'/>"
	"    <menuitem action='20ds'/>"
	"    <menuitem action='40ds'/>"
	"    <menuitem action='60ds'/>"
	"    <menuitem action='80ds'/>"
	"    <menuitem action='100ds'/>"
	"    <menuitem action='200ds'/>"
	"    <menuitem action='400ds'/>"
	"    <separator/>"
	"    <menuitem action='Manualds'/>"
	"	 </menu>"
	"	 <menu action='Connections'>"
	"    <menuitem action='Unlimitedc'/>"
	"    <separator/>"
	"    <menuitem action='10c'/>"
	"    <menuitem action='20c'/>"
	"    <menuitem action='40c'/>"
	"    <menuitem action='60c'/>"
	"    <menuitem action='80c'/>"
	"    <menuitem action='100c'/>"
	"    <menuitem action='200c'/>"
	"    <menuitem action='400c'/>"
	"    <separator/>"
	"    <menuitem action='Manualc'/>"
	"	 </menu>"
	"	 <menu action='UploadSlots'>"
	"    <menuitem action='Unlimitedst'/>"
	"    <separator/>"
	"    <menuitem action='5st'/>"
	"    <menuitem action='10st'/>"
	"    <menuitem action='15st'/>"
	"    <menuitem action='25st'/>"
	"    <menuitem action='35st'/>"
	"    <menuitem action='50st'/>"
	"    <menuitem action='75st'/>"
	"    <menuitem action='100st'/>"
	"    <separator/>"
	"    <menuitem action='Manualst'/>"
	"	 </menu>"	
	"    <separator/>"
	"    <menuitem action='ResumeTorrents'/>"
	"    <menuitem action='StopTorrents'/>"
	"    <separator/>"
	"    <menuitem action='Quit'/>"	
	"  </popup>"
    "</ui>";
	
#ifdef GLIBMM_EXCEPTIONS_ENABLED
	try{
		s_refUIManager->add_ui_from_string(s_ui_info);
	}
	catch(const Glib::Error& ex){
		std::cerr << "building menus failed: " <<  ex.what();
	}
#else
	std::auto_ptr<Glib::Error> ex;
	s_refUIManager->add_ui_from_string(s_ui_info, ex);
	if(ex.get()){
		std::cerr << "building menus failed: " <<  ex->what();
	}
#endif //GLIBMM_EXCEPTIONS_ENABLED
	
}

void SystemTray::toggleShowButton(){
	showLince->set_active(!showLince->get_active());
}

void SystemTray::on_my_popup_menu(guint button, guint32 time){
	Gtk::Menu* pMenu = static_cast<Gtk::Menu*>( s_refUIManager->get_widget("/PopupMenu") );
    systemTray->popup_menu_at_position(*pMenu, button, time);
}

void SystemTray::setToolTip(float uS, float dS, int downloading, int downloaded){
	Glib::ustring toolTip;
	std::ostringstream tempStr;
	
	toolTip = _("D: ") + size_to_string(dS) + " " + _("U: ") + size_to_string(uS);
	
	tempStr << std::endl << _("Downloading: ") << downloading << std::endl <<\
		_("Seeding: ") << downloaded;
	
	toolTip += tempStr.str();
	
	systemTray->set_tooltip(toolTip);
}

SystemTray::~SystemTray(){
}
