/***************************************************************************
 *            Utils.cc
 *
 *  Sun Jan 21 00:33:34 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include <stdlib.h>
#include <unistd.h>

#include "Utils.h"

#include "Lince/Utils.h"

#include <fstream>
#include <cctype>
#include <algorithm>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/regex.hpp>

#include "libtorrent/ip_filter.hpp"

#include "../config.h"
#include <glibmm/i18n.h>

namespace Utils{
	
	Glib::ustring getFilenameFromSaveDialog(const Glib::ustring & message, const Glib::ustring & filename, const Glib::ustring & directory){
		int result;
		Gtk::FileChooserDialog dialog(message, Gtk::FILE_CHOOSER_ACTION_SAVE);
		
		if (!filename.empty())	dialog.set_current_name(filename);
		if (!directory.empty()) dialog.set_current_folder(directory);
		
		//Add response buttons the the dialog.
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
		result = dialog.run();
	
		//Handle the response:
		if (result == Gtk::RESPONSE_OK)
			return dialog.get_filename();

		return "";
	}

	Glib::ustring getFilenameFromLoadDialog(const Glib::ustring & message, const Glib::ustring & directory, const Glib::ustring & pattern_name, const Glib::ustring & pattern){
		int result;
		Gtk::FileChooserDialog dialog(message, Gtk::FILE_CHOOSER_ACTION_OPEN);
		Gtk::FileFilter fileFilter, allFilter;
	
		if (!pattern_name.empty()){
			fileFilter.set_name(pattern_name);
			fileFilter.add_pattern(pattern);
		}
	
		allFilter.set_name(_("All files"));
		allFilter.add_pattern("*");
	
		if (!pattern_name.empty()) dialog.add_filter(fileFilter);
		dialog.add_filter(allFilter);
		
		if (!directory.empty()) dialog.set_current_folder(directory);
		
		//Add response buttons the the dialog.
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
		result = dialog.run();
	
		//Handle the response:
		if (result == Gtk::RESPONSE_OK)
			return dialog.get_filename();

		return "";
	}
	
	std::vector<Glib::ustring> getFilenamesFromLoadDialog(const Glib::ustring & message, const Glib::ustring & directory, const Glib::ustring & pattern_name, const Glib::ustring & pattern){
		int result;
		Gtk::FileChooserDialog dialog(message, Gtk::FILE_CHOOSER_ACTION_OPEN);
		Gtk::FileFilter fileFilter, allFilter;
		std::vector<Glib::ustring> selectedFiles;
	
		if (!pattern_name.empty()){
			fileFilter.set_name(pattern_name);
			fileFilter.add_pattern(pattern);
		}
	
		allFilter.set_name(_("All files"));
		allFilter.add_pattern("*");
	
		dialog.set_select_multiple();
		if (!pattern_name.empty()) dialog.add_filter(fileFilter);
		dialog.add_filter(allFilter);
		
		if (!directory.empty()) dialog.set_current_folder(directory);
		
		//Add response buttons the the dialog.
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
		result = dialog.run();
	
		//Handle the response:
		if (result == Gtk::RESPONSE_OK)
			selectedFiles = dialog.get_filenames();

		return selectedFiles;
	}
	
	
	Glib::ustring getDirectoryFromDialog(const Glib::ustring & message, const Glib::ustring & directory){
		int result;
		Gtk::FileChooserDialog dialog(message, Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
		
		if (!directory.empty()) dialog.set_current_folder(directory);

		//Add response buttons the the dialog.
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
		result = dialog.run();
	
		//Handle the response:
		if (result == Gtk::RESPONSE_OK)
			return dialog.get_filename();

		return "";
	}
	
	Glib::ustring getDirectoryFromDialog(const Glib::ustring & message){
		int result;
		Gtk::FileChooserDialog dialog(message, Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
		
		//Add response buttons the the dialog.
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
	
		result = dialog.run();
	
		//Handle the response:
		if (result == Gtk::RESPONSE_OK)
			return dialog.get_filename();

		return "";
	}

	
	void warningDialog(const Glib::ustring & header, const Glib::ustring & message){
		Glib::ustring header_message = "<big><big>" + header + "</big></big>";
		Gtk::MessageDialog warning_window(header_message,  true, Gtk::MESSAGE_WARNING);
		
		warning_window.set_secondary_text(message);
	
		warning_window.set_title(_("Warning!"));
		warning_window.show();
		warning_window.run();
	}
	
	bool askDialog(const Glib::ustring & title, const Glib::ustring & header, const Glib::ustring & message){
		Glib::ustring header_message = "<big><big>" + header + "</big></big>";
		int result;
		
		Gtk::MessageDialog ask_dialog(header_message,  true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
		
		ask_dialog.set_title(title);
		ask_dialog.set_secondary_text(message);
		
		result = ask_dialog.run();
		
		if (result == Gtk::RESPONSE_YES){
			return true;
		}
		
		return false;
	}
	
	int askDialogYesNoCancel(const Glib::ustring & title, const Glib::ustring & header, const Glib::ustring & message){
		Glib::ustring header_message = "<big><big>" + header + "</big></big>";
		int result;
		
		Gtk::MessageDialog ask_dialog(header_message,  true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
		
		ask_dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		ask_dialog.add_button(Gtk::Stock::NO, Gtk::RESPONSE_NO);
		ask_dialog.add_button(Gtk::Stock::YES, Gtk::RESPONSE_YES);
		
		ask_dialog.set_title(title);
		ask_dialog.set_secondary_text(message);
		
		result = ask_dialog.run();
		
		return result;
	}
	
	bool askIntValueDialog(const Glib::ustring & message, const Glib::ustring & label, int & value){
		int result;
		Gtk::Dialog dialog(message);
		Gtk::Label valueLabel(label);
		Gtk::Entry valueEntry;
		
		dialog.get_vbox()->pack_start(valueLabel);
		dialog.get_vbox()->pack_start(valueEntry);
		dialog.get_vbox()->show_all();
		
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
		
		result = dialog.run();
		
		if (result == Gtk::RESPONSE_OK){
			value = Lince::to_int(valueEntry.get_text());
			return true;
		}
		
		value = 0;
		return false;
	}
	
	bool askUstringDialog(const Glib::ustring & message, const Glib::ustring & label, Glib::ustring & answer){
		int result;
		Gtk::Dialog dialog(message);
		Gtk::Label valueLabel(label);
		Gtk::Entry valueEntry;
		
		if (!answer.empty()) valueEntry.set_text(answer);
		
		dialog.get_vbox()->pack_start(valueLabel);
		dialog.get_vbox()->pack_start(valueEntry);
		dialog.get_vbox()->show_all();
		
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
		
		result = dialog.run();
		
		if (result == Gtk::RESPONSE_OK){
			answer = valueEntry.get_text();
			return true;
		}
		
		answer = "";
		return false;
	}
	
	Glib::ustring askNameDialog(const Glib::ustring & message, const Glib::ustring & _name){
		int result;
		Glib::ustring name;
		Gtk::Dialog dialog(message);
		Gtk::Label valueLabel(_("Name:"));
		Gtk::Entry valueEntry;
		Gtk::HBox hbox;
		
		if (!_name.empty()) valueEntry.set_text(_name);
		
		hbox.pack_start(valueLabel, Gtk::PACK_SHRINK, 10);
		hbox.pack_start(valueEntry, Gtk::PACK_EXPAND_WIDGET, 5);
		
		dialog.get_vbox()->add(hbox);
		
		dialog.get_vbox()->show_all();
		
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
		
		result = dialog.run();
		
		if (result == Gtk::RESPONSE_OK){
			name = valueEntry.get_text();
		}
		
		return name;
	}
		
		
	
	bool askLoginDataDialog(const Glib::ustring & message, std::pair<Glib::ustring, Glib::ustring> &loginData){
		int result;
		Gtk::Dialog dialog(message);
		Gtk::Label loginLabel(_("Username:")), passwordLabel(_("Password:"));
		Gtk::Entry loginEntry, passwordEntry;
		Gtk::Table table;
		
		passwordEntry.set_visibility(false);
		
		table.attach(loginLabel, 0, 1, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 10, 5);
		table.attach(loginEntry, 1, 2, 0, 1, Gtk::FILL, Gtk::SHRINK, 10, 5);
		table.attach(passwordLabel, 0, 1, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 10, 5);
		table.attach(passwordEntry, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 10, 5);
		
		dialog.get_vbox()->pack_start(table, Gtk::PACK_EXPAND_WIDGET);		
		dialog.get_vbox()->show_all();
		
		dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
		
		result = dialog.run();
		
		if (result == Gtk::RESPONSE_OK){
			if (!loginEntry.get_text_length() || !passwordEntry.get_text_length()) 
				return false;
			loginData.first = loginEntry.get_text();
			loginData.second = passwordEntry.get_text();
			return true;
		}
		
		loginData.first = "";
		loginData.second = "";
		return false;
	}
	
	bool isAnURL(Glib::ustring &url){
		//boost::regex ex("((https?):((//)|(\\\\))+[\\w\\d:#@%/;$()~_?\\+-=\\\\.&]*)");
		//boost::cmatch what;
		
		/*if (boost::regex_match(url.c_str(), what, ex) && (url != "http://")){
			return true;
		}*/
#if !(defined(_WIN32) && defined(__MINGW32__))
		Glib::RefPtr<Glib::Regex> ex = Glib::Regex::create("((https?):((//)|(\\\\))+[\\w\\d:#@%/;$()~_?\\+-=\\\\.&]*)");
		
		if (ex->match(url) && (url != "http://"))
			return true;
		
		return false;
#else
		return true;
#endif
	}
	
	void registerIcon(const Glib::RefPtr<Gtk::IconFactory>& factory, const std::string& fname, const Glib::ustring& id, const Glib::ustring& label){
		Gtk::IconSource source;
		Glib::ustring filepath = Glib::build_filename(ICON_DIR, fname);
		
		try{
			//This throws an exception if the file is not found:
			source.set_pixbuf( Gdk::Pixbuf::create_from_file(filepath) );
		}
		catch(const Glib::Exception& ex){
			std::cout << ex.what() << std::endl;
		}
	
		source.set_size(Gtk::ICON_SIZE_SMALL_TOOLBAR);
		source.set_size_wildcarded(); //Icon may be scaled.
	
		Gtk::IconSet icon_set;
		icon_set.add_source(source); 
	
		const Gtk::StockID stock_id(id);
  		factory->add(stock_id, icon_set);
  		Gtk::Stock::add(Gtk::StockItem(stock_id, label));
	}
	
	void uSleep(int64_t value){
#if !(defined(_WIN32) && defined(__MINGW32__))
		usleep(value);
#else
		Sleep(value/1000);
#endif
	}
}
